/**
 * Copyright (C) 2012 Stephen M. Hopper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.enragedginger.stephenerialization;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.enragedginger.stephenerialization.destephenerialize.Destephenerializer;
import com.enragedginger.stephenerialization.stephenerialize.Stephenerializer;
import com.enragedginger.stephenerialization.streamer.DestephenerializationStream;
import com.enragedginger.stephenerialization.streamer.SimpleDestephenerializationStream;
import com.enragedginger.stephenerialization.streamer.SimpleStephenerializationStream;
import com.enragedginger.stephenerialization.streamer.StephenerializationStream;
import com.google.inject.Inject;

/**
 * A simple implementation of {@link StephenerializationService}.
 * @author Stephen Hopper
 *
 */
public class SimpleStephenerializationService implements
		StephenerializationService {
	
	private Stephenerializer stephenerializer;
	
	private Destephenerializer destephenerializer;

	/**
	 * {@inheritDoc}
	 */
	public void stephenerialize(Object object, ObjectOutputStream streamer) {
		stephenerializer.stephenerialize(object, createStream(streamer));
	}

	/**
	 * {@inheritDoc}
	 */
	public void destephenerialize(Object object, ObjectInputStream streamer) {
		destephenerializer.destephenerialize(object, createStream(streamer));
	}

	/**
	 * @param stephenerializer The stephenerializer to use.
	 */
	@Inject
	public void setStephenerializer(Stephenerializer stephenerializer) {
		this.stephenerializer = stephenerializer;
	}

	/**
	 * @param destephenerializer The destephenerializer to use.
	 */
	@Inject
	public void setDestephenerializer(Destephenerializer destephenerializer) {
		this.destephenerializer = destephenerializer;
	}
	
	/**
	 * Creates a {@link StephenerializationStream}.
	 * @param streamer The stream to wrap.
	 * @return The {@link StephenerializationStream}.
	 */
	protected StephenerializationStream createStream(ObjectOutputStream streamer) {
		return new SimpleStephenerializationStream(streamer);
	}
	
	/**
	 * Creates a {@link DestephenerializationStream}.
	 * @param streamer The stream to wrap.
	 * @return The {@link DestephenerializationStream}.
	 */
	protected DestephenerializationStream createStream(ObjectInputStream streamer) {
		return new SimpleDestephenerializationStream(streamer);
	}

}
